/************************************************************************
 * 			MEKATRONIX Copyright 1998 		        *
 * Title        avoidtjp.c                                              *
 * Programmer	Keith L. Doty and modified for IC by Scott Jantz     	*
 * Date         August 25, 1998                                         *
 * Version	1			                		*
 *								        *
 * Description								*
 *    A very simple collision avoidance program. TJ PRO will read 	*
 *    each IR detectors, and turn away from any obstacles in its	*
 *    path. Also, if something hits TJ PRO's bumper, it will back up,	*
 *    turn, and go on.							*
 *			                                                *     					*
 ************************************************************************/
 

/*************************** Includes ********************************/

#include <tjpbase.h>
#include <stdio.h>

/************************ End of Includes ****************************/


/*************************** Constants ********************************/

#define AVOID_THRESHOLD 87			/* Modified from 100 by Jeffrey */ 
#define LOWER_AVOID_THRESH 85
#define MAX_SPEEDL 70
#define MAX_SPEEDR 70
/************************ End of Constants ****************************/


/*************************** Prototypes *********************************/
void turn(void);
/************************ End of Prototypes *****************************/

void main(void)
/****************************** Main ***********************************/
{
  int irdr, irdl, speedr, speedl;
  int flag;
  init_analog();
  init_motortjp();
  init_clocktjp();

  IRE_ON;     /* turn on IR emitters */


  START;  /*Press the rear bumper to start the program*/
  flag = 0;
   while(1)
  { 

/*
   The following block will read the IR detectors, and decide whether TJ
   needs to turn to avoid any obstacles
*/
    irdr = RIGHT_IR;
    irdl = LEFT_IR;
	 speedr = 0;
	speedl = 0;
	if ((irdl >= LOWER_AVOID_THRESH) & (irdl <= AVOID_THRESHOLD) & (irdr >= LOWER_AVOID_THRESH) & (irdr <= AVOID_THRESHOLD))
	{ speedr = MAX_SPEEDR;
	  speedl = MAX_SPEEDL;
	  wait(27);}
			
	else
	{
		if ((irdr > AVOID_THRESHOLD) &	(irdl >= LOWER_AVOID_THRESH) & (irdl <= AVOID_THRESHOLD))
		{ 	speedr = 0;
		  	speedl = MAX_SPEEDL;
		  	motorp(RIGHT_MOTOR, speedr); 
    	  	motorp(LEFT_MOTOR, speedl);
    	  	wait(27);
    	  	speedr = MAX_SPEEDR;
	      	speedl = MAX_SPEEDL;}
		if ((irdr < LOWER_AVOID_THRESH) &	(irdl >= LOWER_AVOID_THRESH) & (irdl <= AVOID_THRESHOLD))	   
		{ 	speedr = MAX_SPEEDR;
		  	speedl = 0;
		    motorp(RIGHT_MOTOR, speedr); 
		    motorp(LEFT_MOTOR, speedl);
    		wait(23);
    	if (irdr < LOWER_AVOID_THRESH)
    	{	speedr = MAX_SPEEDR;
		  	speedl = 0;
		    motorp(RIGHT_MOTOR, speedr); 
    		motorp(LEFT_MOTOR, speedl);
    	    wait(23);}
    	    speedr = MAX_SPEEDR;
   			speedl = MAX_SPEEDL;}
		else 
		{   flag = 0;
			speedr = MAX_SPEED;
			speedl = MAX_SPEED;
			wait(23);}
	}
	if ((irdl > AVOID_THRESHOLD) &	(irdr >= LOWER_AVOID_THRESH) & (irdr <= AVOID_THRESHOLD))
		{ speedr = MAX_SPEEDR;
		  speedl = 0;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(23);
    	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;}
	if ((irdl < LOWER_AVOID_THRESH) &	(irdr >= LOWER_AVOID_THRESH) & (irdr <= AVOID_THRESHOLD))	   
	{ 	  speedr = 0;
		  speedl = MAX_SPEEDL;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(27);
    	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;}	  
	if ((irdl < LOWER_AVOID_THRESH) & (irdr >= 86 ))
	{ 	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(70);
    	  speedr = -MAX_SPEED;
		  speedl = MAX_SPEED;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(25);
    	  flag = 1;
	}
	if ((irdr < LOWER_AVOID_THRESH) & (irdl >= 86))
	{ 	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(70);
    	  speedr = MAX_SPEED;
		  speedl = -MAX_SPEED;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(25);
		  flag = 1;
	}
	if ((irdr < LOWER_AVOID_THRESH) & (irdl < LOWER_AVOID_THRESH))
	{ 	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(100);
    	  speedr = -MAX_SPEED;
		  speedl = MAX_SPEED;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(25);  
		  flag = 1;
	}
	if ((irdl > AVOID_THRESHOLD)  || (irdr > AVOID_THRESHOLD))
	{	  speedr = 0;
		  speedl = -MAX_SPEED;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(25);
    	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(70);
	}
		
    motorp(RIGHT_MOTOR, speedr); 
    motorp(LEFT_MOTOR, speedl);

/* This "if" statement checks the bumper. If the bumper is pressed, */
/* Tj will back up, and turn.                                       */

    if((FRONT_BUMP) || (FRONT_IR > AVOID_THRESHOLD))
    {
      motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(450);
    }
    if((RIGHT_BUMP))
    {
      motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(250);
    }
    

  }    
}
/**************************** End of Main ******************************/

void turn(void)
/************************************************************************
 * Function:  Will turn in a random direction for a "random" amount of  *
 *            time, dictated by the fast changine lower bits in		*
 *	      mseconds().  					        *
 * Returns:  None                                                       *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  TCNT                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Functions called: motorp(), wait()					*
 * Notes:                                                               *
 ************************************************************************/
{
 int i;
 unsigned rand;

 rand = TCNT;

 if (rand & 0x0001)
 /*turn left*/
 {
   motorp(RIGHT_MOTOR, MAX_SPEED);
   motorp(LEFT_MOTOR, -MAX_SPEED);
 }
 else
 /*turn right*/
 {
   motorp(RIGHT_MOTOR, -MAX_SPEED);
   motorp(LEFT_MOTOR, MAX_SPEED);
 }

 i=(rand % 1024);
 if(i>250) wait(i); else wait(250);

}

/***********************End Function turn ****************************/


