/************************************************************************
 * 			MEKATRONIX Copyright 1998 		        *
 * Title        avoidtjp.c                                              *
 * Programmer	Keith L. Doty and modified for IC by Scott Jantz     	*
 * Date         August 25, 1998                                         *
 * Version	1			                		*
 *								        *
 * Description								*
 *    A very simple collision avoidance program. TJ PRO will read 	*
 *    each IR detectors, and turn away from any obstacles in its	*
 *    path. Also, if something hits TJ PRO's bumper, it will back up,	*
 *    turn, and go on.							*
 *			                                                *     					*
 ************************************************************************/
 

/*************************** Includes ********************************/

#include <tjpbase.h>
#include <stdio.h>
#include <linef.h>

/************************ End of Includes ****************************/


/*************************** Constants ********************************/

#define AVOID_THRESHOLD 90			/* Modified from 100 by Jeffrey */ 
#define LOWER_AVOID_THRESH 87
#define MAX_SPEEDL -70
#define MAX_SPEEDR -70
#define SMOOTH_SPEED -100
#define ERR 0
#define ERRM 35
#define TURNT 800
#define ACCEL_RATE 20
#define MAX_ACCEL 35
#define MIN_ACCEL 15
#define MIN_SPEED 15
#define MAX_SPEED2 20

/************************ End of Constants ****************************/


/*************************** Prototypes *********************************/
void turn(void);
void smooth(void);
void wallfollow(void);
void linefollow(void);
void read_cds(void);
void run_motor(int speedr, int speedl, int time);

int irdr, irdl, speedr, speedl;
 int temp7, temp8,temp9, temp10;
  int initp1, initp2, initpl1, initpl2, accel, off_left, off_right, temp1, temp2,temp3, temp4, temp5,  curp1, curp2, flag;
  int curpl1, curpl2;
  int slow_down, lost_count, line_now;

/************************ End of Prototypes *****************************/

void main(void)
/****************************** Main ***********************************/
{
  int irdr, irdl, speedr, speedl;
  int flag, flag2;
  init_analog();
  init_motortjp();
  init_clocktjp();

  IRE_ON;     /* turn on IR emitters */


 	line_now = 0;
  flag = 0;
  temp1 = PHOTO1;
	wait(50);
	temp2 = PHOTO1;
	wait(50);
	temp3 = PHOTO1;
	wait(50);
	temp4 = PHOTO1;
	initp1 = ((temp1 + temp2 + temp3 + temp4)/4);
	wait(50);
	temp1 = PHOTO2;
	wait(50);
	temp2 = PHOTO2;
	wait(50);
	temp3 = PHOTO2;
	wait(50);
	temp4 = PHOTO2;
	initp2 = ((temp1 + temp2 + temp3 + temp4)/4);
	wait(50);
	temp1 = PHOTOL1;
	wait(50);
	temp2 = PHOTOL1;
	wait(50);
	temp3 = PHOTOL1;
	wait(50);
	temp4 = PHOTOL1;
	initpl1 = ((temp1 + temp2 + temp3 + temp4)/4);
	wait(50);
	temp1 = PHOTOL2;
	wait(50);
	temp2 = PHOTOL2;
	wait(50);
	temp3 = PHOTOL2;
	wait(50);
	temp4 = PHOTOL2;
	initpl2 = ((temp1 + temp2 + temp3 + temp4)/4);
	START;  /*Press the rear bumper to start the program*/
	wait(200);
   while(1)
  { 
	smooth();
	//wallfollow();
	//if (line_now == 0) wallfollow();
	//else linefollow();


  }
}
/**************************** End of Main ******************************/



void smooth(void)
{
int irdr, irdl, irfl, irfr, speedr, speedl;
  int flag, flag2;
flag2 = 0;

   while(1)
   
  { 
/*
   The following block will read the IR detectors, and decide whether TJ
   needs to turn to avoid any obstacles
*/
    irdr = SIDE_RIGHT_IR;
    irdl = SIDE_LEFT_IR;
    irfl = FRONT_LEFT_IR;
    irfr = FRONT_RIGHT_IR;

    if (LEFT_BUMP || BACK_BUMP)
      speedr = -SMOOTH_SPEED;
    else
      speedr = SMOOTH_SPEED;
    if ((irdr > AVOID_THRESHOLD) || (irdl > AVOID_THRESHOLD))
    { lost_count = 0;
      wallfollow();}
    else
    {  speedl = SMOOTH_SPEED;}
    
    motorp(RIGHT_MOTOR, speedr); 
    motorp(LEFT_MOTOR, speedl);
								
/* This "if" statement checks the bumper. If the bumper is pressed, */
/* Tj will back up, and turn.                                       */

    if(FRONT_BUMP || irfr > AVOID_THRESHOLD || irfl > AVOID_THRESHOLD)
    {
      if (flag2 == 0)
      {
      	motorp(LEFT_MOTOR, -SMOOTH_SPEED);
      	motorp(RIGHT_MOTOR, -SMOOTH_SPEED);
      	wait(400);
      	motorp(LEFT_MOTOR, -SMOOTH_SPEED);
      	motorp(RIGHT_MOTOR, SMOOTH_SPEED);
      	wait (400);
      	motorp(LEFT_MOTOR, SMOOTH_SPEED);
      	motorp(RIGHT_MOTOR, SMOOTH_SPEED);
      	wait(600);
      	motorp(LEFT_MOTOR, -SMOOTH_SPEED);
      	motorp(RIGHT_MOTOR, SMOOTH_SPEED);
      	wait (375);
      	flag2 = 1;
      }
      else
      {	motorp(LEFT_MOTOR, -SMOOTH_SPEED);
      	motorp(RIGHT_MOTOR, -SMOOTH_SPEED);
      	wait(400);
      	motorp(LEFT_MOTOR, SMOOTH_SPEED);
      	motorp(RIGHT_MOTOR,- SMOOTH_SPEED);
      	wait (425);
      	motorp(LEFT_MOTOR, SMOOTH_SPEED);
      	motorp(RIGHT_MOTOR, SMOOTH_SPEED);
      	wait(600);
      	motorp(LEFT_MOTOR, SMOOTH_SPEED);
      	motorp(RIGHT_MOTOR, -SMOOTH_SPEED);
      	wait (450);
      	flag2 = 0;
      }
 
    }

  } 
  return;
}

void wallfollow(void)
{
int irdr, irdl, speedr, speedl;
  int flag, flag2;
int lost_count;
flag = 0;
lost_count = 0;
   while(1)
  { 
	if (lost_count > 30) {lost_count = 0; break;}
/*
   The following block will read the IR detectors, and decide whether TJ
   needs to turn to avoid any obstacles
*/
    irdr = SIDE_RIGHT_IR;
    irdl = SIDE_LEFT_IR;
	 speedr = 0;
	speedl = 0;
	if ((irdl >= LOWER_AVOID_THRESH) & (irdl <= AVOID_THRESHOLD) & (irdr >= LOWER_AVOID_THRESH) & (irdr <= AVOID_THRESHOLD))
	{ speedr = MAX_SPEEDR;
	  speedl = MAX_SPEEDL;
	  wait(27);}
			
	else
	{
		if ((irdr > AVOID_THRESHOLD) &	(irdl >= LOWER_AVOID_THRESH) & (irdl <= AVOID_THRESHOLD))
		{ 	speedr = 0;
		  	speedl = MAX_SPEEDL;
		  	motorp(RIGHT_MOTOR, speedr); 
    	  	motorp(LEFT_MOTOR, speedl);
    	  	wait(27);
    	  	speedr = MAX_SPEEDR;
	      	speedl = MAX_SPEEDL;
	      	lost_count = 0;}
		if ((irdr < LOWER_AVOID_THRESH) &	(irdl >= LOWER_AVOID_THRESH) & (irdl <= AVOID_THRESHOLD))	   
		{ 	speedr = MAX_SPEEDR;
		  	speedl = 0;
		    motorp(RIGHT_MOTOR, speedr); 
		    motorp(LEFT_MOTOR, speedl);
    		wait(23);
    	if (irdr < LOWER_AVOID_THRESH)
    	{	speedr = MAX_SPEEDR;
		  	speedl = 0;
		    motorp(RIGHT_MOTOR, speedr); 
    		motorp(LEFT_MOTOR, speedl);
    	    wait(23);}
    	    speedr = MAX_SPEEDR;
   			speedl = MAX_SPEEDL;
   			lost_count = 0;}
		else 
		{   flag = 0;
			speedr = MAX_SPEED;
			speedl = MAX_SPEED;
			wait(23);}
			}
	if ((irdl > AVOID_THRESHOLD) &	(irdr >= LOWER_AVOID_THRESH) & (irdr <= AVOID_THRESHOLD))
		{ speedr = MAX_SPEEDR;
		  speedl = 0;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(23);
    	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;
		  lost_count = 0;}
	if ((irdl < LOWER_AVOID_THRESH) &	(irdr >= LOWER_AVOID_THRESH) & (irdr <= AVOID_THRESHOLD))	   
	{ 	  speedr = 0;
		  speedl = MAX_SPEEDL;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(27);
    	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;
		  lost_count = 0;}	  
	if ((irdl < LOWER_AVOID_THRESH) & (irdr >= 86 ))
	{ 	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(70);
    	  speedr = -MAX_SPEED;
		  speedl = MAX_SPEED;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(25);
    	  flag = 1;
    	  lost_count = 0;
	}
	if ((irdr < LOWER_AVOID_THRESH) & (irdl >= 86))
	{ 	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(70);
    	  speedr = MAX_SPEED;
		  speedl = -MAX_SPEED;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(25);
		  flag = 1;
		  lost_count = 0;
	}
	if ((irdr < LOWER_AVOID_THRESH) & (irdl < LOWER_AVOID_THRESH))
	{ 	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(100);
    	  speedr = -MAX_SPEED;
		  speedl = MAX_SPEED;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(25);  
		  flag = 1;
		 lost_count = lost_count + 1;
	}
	if ((irdl > AVOID_THRESHOLD)  || (irdr > AVOID_THRESHOLD))
	{	  speedr = 0;
		  speedl = -MAX_SPEED;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(25);
    	  speedr = MAX_SPEEDR;
		  speedl = MAX_SPEEDL;
		  motorp(RIGHT_MOTOR, speedr); 
    	  motorp(LEFT_MOTOR, speedl);
    	  wait(70);
    	  lost_count = 0;
	}
		
    motorp(RIGHT_MOTOR, speedr); 
    motorp(LEFT_MOTOR, speedl);

/* This "if" statement checks the bumper. If the bumper is pressed, */
/* Tj will back up, and turn.                                       */

    if((FRONT_BUMP) || (FRONT_LEFT_IR > AVOID_THRESHOLD) || (FRONT_RIGHT_IR > AVOID_THRESHOLD))
    { motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, -MAX_SPEED);
      wait(150);
      motorp(LEFT_MOTOR, MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(75);
      motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(450);
    }
    if((RIGHT_BUMP))
    {
      motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(250);
    }
    

  }
  return;
}   

void linefollow(void)
{


	
	flag = 0;
	speedr = 20;
	speedl = 20;
	off_left = 0;
	lost_count = 0;
	off_right = 0;
	accel= ACCEL_RATE;
	
while(1)
  {	read_cds();
	if (lost_count > 10) break;
	if((FRONT_BUMP) || (FRONT_LEFT_IR > AVOID_THRESHOLD))
    { motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, -MAX_SPEED);
      wait(150);
      motorp(LEFT_MOTOR, MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(75);
      motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(450);
      break;
    }
	if ((flag == 0) & P_ALL_ON))
	
	{	lost_count = 0;
		off_left = 0;
		off_right = 0;
		read_cds();
		speedr = speedl;
		speedr = speedr + 1;
		speedl = speedr + 1;
		accel = accel+ 4;
		if (accel > MAX_ACCEL) accel = MAX_ACCEL;
		if ((speedr == MAX_SPEED2) | (speedl = MAX_SPEED2)) {speedr = MAX_SPEED2; speedl = MAX_SPEED2;	}
		run_motor(speedr, speedl, accel);
    	flag = 0;	
    	
	}
	

	while ((flag == 0) & (P_LFT_OFF | (P_LFT_OFF & P_FNT_OFF)))
	{		if((FRONT_BUMP) || (FRONT_LEFT_IR > AVOID_THRESHOLD))
    { motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, -MAX_SPEED);
      wait(150);
      motorp(LEFT_MOTOR, MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(75);
      motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(450);
      break;
    }

			if ((P_RGT_OFF & P_LFT_OFF & P_FNT_OFF))
    		{	lost_count = lost_count + 1;
    		}	    
    		else lost_count = 0;
			accel = accel - 3;
			off_left = 1;
			off_right = 0;
			read_cds();
			speedr = speedr - 1;
       		speedl = speedl + 1;
       		if (accel < MIN_ACCEL) accel = MIN_ACCEL;
       		if ((speedr <= MIN_SPEED + 10) | (speedl <= MIN_SPEED + 10)) {speedr = MIN_SPEED + 10;}
       		if ((speedl >= MAX_SPEED2 + 15)) {speedl = MAX_SPEED2 + 15;}
			run_motor(speedr, speedl, accel);
    		flag = 0;	

  
    	
    }
   while ((flag == 0) & (P_RGT_OFF | (P_RGT_OFF & P_FNT_OFF)))
	{		if((FRONT_BUMP) || (FRONT_LEFT_IR > AVOID_THRESHOLD))
    { motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, -MAX_SPEED);
      wait(150);
      motorp(LEFT_MOTOR, MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(75);
      motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, MAX_SPEED);
      wait(450);
      break;
    }

			if ((P_RGT_OFF & P_LFT_OFF & P_FNT_OFF))
    		{	lost_count = lost_count + 1;
    		}  
    		else lost_count = 0;
			off_left = 0;
			off_right = 1;
			read_cds();
			speedr = speedr + 2;
       		speedl = speedl - 2;
       		accel = accel - 3;
       		if (accel < MIN_ACCEL) accel = MIN_ACCEL;
       		if ((speedr <= MIN_SPEED + 10) | (speedl <= MIN_SPEED) + 10) {speedl = MIN_SPEED + 10;}
       		if ((speedr >= MAX_SPEED2 + 12)) {speedr = MAX_SPEED2 + 12;}
			run_motor(speedr, speedl, accel);
    		flag = 0;
    	
    }
    

			if ((P_RGT_OFF & P_LFT_OFF & P_FNT_OFF))
    		{	lost_count = lost_count + 1;
    		}  
  }/*end while(1)*/
}

void run_motor(int speedr, int speedl, int time)
{	motorp(RIGHT_MOTOR, speedr); 
   	motorp(LEFT_MOTOR, speedl);
   	wait(time);}
   	
void read_cds(void)
{
temp1 = PHOTO1;
  	if (temp1 < initp1-ERRM) temp1 = PHOTO1;
	temp2 = PHOTO1;
	if (temp2 < initp1-ERRM) temp2 = PHOTO1;
	temp3 = PHOTO1;
	if (temp3 < initp1-ERRM) temp3 = PHOTO1;
	temp4 = PHOTO1;
	if (temp4 < initp1-ERRM) temp4 = PHOTO1;
	curp1 = ((temp1 + temp2 + temp3 + temp4)/4);
	temp1 = PHOTO2;
 	if (temp1 < initp2-ERRM) temp1 = PHOTO2;
	temp2 = PHOTO2;
	if (temp2 < initp2-ERRM) temp2 = PHOTO2;
	temp3 = PHOTO2;
	if (temp3 < initp2-ERRM) temp3 = PHOTO2;
	temp4 = PHOTO2;
	if (temp4 < initp2-ERRM) temp4 = PHOTO2;
	curp2 = ((temp1 + temp2 + temp3 + temp4)/4);
	temp1 = PHOTOL1;
	if (temp1 < initpl1-ERRM) temp1 = PHOTOL1;
	temp2 = PHOTOL1;
	if (temp2 < initpl1-ERRM) temp2 = PHOTOL1;
	temp3 = PHOTOL1;
	if (temp3 < initpl1-ERRM) temp3 = PHOTOL1;
	temp4 = PHOTOL1;
	if (temp4 < initpl1-ERRM) temp4 = PHOTOL1;
	curpl1 = ((temp1 + temp2 + temp3 + temp4)/4);
	temp1 = PHOTOL2;
 	if (temp1 < initpl2-ERRM) temp1 = PHOTOL2;
	temp2 = PHOTOL2;
	if (temp2 < initpl2-ERRM) temp2 = PHOTOL2;
	temp3 = PHOTOL2;
	if (temp3 < initpl2-ERRM) temp3 = PHOTOL2;
	temp4 = PHOTOL2;
	if (temp4 < initpl2-ERRM) temp4 = PHOTOL2;
	curpl2 = ((temp1 + temp2 + temp3 + temp4)/4);}